@@ -93,11 +93,13 @@
 
 #include "local.h"
 
+#define STR(x) vstring_str(x)
+
 /* deliver_switch - branch on recipient type */
 
 static int deliver_switch(LOCAL_STATE state, USER_ATTR usr_attr)
 {
-    const char *myname = "deliver_switch";
+    char   *myname = "deliver_switch";
     int     status = 0;
     struct stat st;
     struct mypasswd *mypwd;
@@ -119,8 +121,8 @@ static int deliver_switch(LOCAL_STATE state, USER_ATTR usr_attr)
      * 
      * XXX Should test for presence of user home directory.
      */
-    if (state.msg_attr.rcpt.address[0] == '\\') {
-	state.msg_attr.rcpt.address++, state.msg_attr.local++, state.msg_attr.user++;
+    if (state.msg_attr.recipient[0] == '\\') {
+	state.msg_attr.recipient++, state.msg_attr.local++, state.msg_attr.user++;
 	if (deliver_mailbox(state, usr_attr, &status) == 0)
 	    status = deliver_unknown(state, usr_attr);
 	return (status);
@@ -181,12 +183,11 @@ static int deliver_switch(LOCAL_STATE state, USER_ATTR usr_attr)
      */
     if (var_stat_home_dir
 	&& (mypwd = mypwnam(state.msg_attr.user)) != 0
-	&& stat_as(mypwd->pw_dir, &st, mypwd->pw_uid, mypwd->pw_gid) < 0) {
-	dsb_simple(state.msg_attr.why, "4.3.0",
-		   "cannot access home directory %s: %m", mypwd->pw_dir);
+	&& stat_as(mypwd->pw_dir, &st, mypwd->pw_uid, mypwd->pw_gid) < 0)
 	return (defer_append(BOUNCE_FLAGS(state.request),
-			     BOUNCE_ATTR(state.msg_attr)));
-    }
+			     BOUNCE_ATTR(state.msg_attr, "4.3.0"),
+			     "cannot access home directory %s: %m",
+			     mypwd->pw_dir));
     if (deliver_dotforward(state, usr_attr, &status) == 0
 	&& deliver_mailbox(state, usr_attr, &status) == 0)
 	status = deliver_unknown(state, usr_attr);
@@ -197,7 +198,7 @@ static int deliver_switch(LOCAL_STATE state, USER_ATTR usr_attr)
 
 int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
 {
-    const char *myname = "deliver_recipient";
+    char   *myname = "deliver_recipient";
     int     rcpt_stat;
 
     /*
@@ -211,7 +212,7 @@ int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
      * Duplicate filter.
      */
     if (been_here(state.dup_filter, "recipient %d %s",
-		  state.level, state.msg_attr.rcpt.address))
+		  state.level, state.msg_attr.recipient))
 	return (0);
 
     /*
@@ -229,7 +230,7 @@ int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
      * need for VERP specific bouncing code, at the cost of complicating the
      * normal bounce sending procedure, but would simplify the code below.
      */
-    if (delivered_find(state.loop_info, state.msg_attr.rcpt.address)) {
+    if (delivered_find(state.loop_info, state.msg_attr.recipient)) {
 	VSTRING *canon_owner = 0;
 
 	if (var_ownreq_special) {
@@ -243,11 +244,11 @@ int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
 	rhs = maps_find(alias_maps, lhs, DICT_FLAG_NONE); \
     }
 
-	    FIND_OWNER(owner_alias, owner_expansion, state.msg_attr.rcpt.address);
+	    FIND_OWNER(owner_alias, owner_expansion, state.msg_attr.recipient);
 	    if (owner_expansion == 0
-	    && (stripped_recipient = strip_addr(state.msg_attr.rcpt.address,
-						(char **) 0,
-						*var_rcpt_delim)) != 0) {
+		&& (stripped_recipient = strip_addr(state.msg_attr.recipient,
+						    (char **) 0,
+						    *var_rcpt_delim)) != 0) {
 		myfree(owner_alias);
 		FIND_OWNER(owner_alias, owner_expansion, stripped_recipient);
 		myfree(stripped_recipient);
@@ -260,15 +261,17 @@ int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
 	    }
 	    myfree(owner_alias);
 	}
-	dsb_simple(state.msg_attr.why, "5.4.6",	"mail forwarding loop for %s",
-		   state.msg_attr.rcpt.address);
 	if (canon_owner) {
 	    rcpt_stat = bounce_one(BOUNCE_FLAGS(state.request),
-				   BOUNCE_ONE_ATTR(state.msg_attr));
+				   BOUNCE_ONE_ATTR(state.msg_attr, "5.4.6"),
+				   "mail forwarding loop for %s",
+				   state.msg_attr.recipient);
 	    vstring_free(canon_owner);
 	} else {
 	    rcpt_stat = bounce_append(BOUNCE_FLAGS(state.request),
-				      BOUNCE_ATTR(state.msg_attr));
+				      BOUNCE_ATTR(state.msg_attr, "5.4.6"),
+				      "mail forwarding loop for %s",
+				      state.msg_attr.recipient);
 	}
 	return (rcpt_stat);
     }
@@ -279,8 +282,8 @@ int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
      * will show the correct forwarding recipient.
      */
     if (state.msg_attr.delivered == 0)
-	state.msg_attr.delivered = state.msg_attr.rcpt.address;
-    state.msg_attr.local = mystrdup(state.msg_attr.rcpt.address);
+	state.msg_attr.delivered = state.msg_attr.recipient;
+    state.msg_attr.local = mystrdup(state.msg_attr.recipient);
     lowercase(state.msg_attr.local);
     if ((state.msg_attr.domain = split_at_right(state.msg_attr.local, '@')) == 0)
 	msg_warn("no @ in recipient address: %s", state.msg_attr.local);
@@ -304,12 +307,10 @@ int     deliver_recipient(LOCAL_STATE state, USER_ATTR usr_attr)
     /*
      * Do not allow null usernames.
      */
-    if (state.msg_attr.user[0] == 0) {
-	dsb_simple(state.msg_attr.why, "5.1.3",
-		   "null username in \"%s\"", state.msg_attr.rcpt.address);
+    if (state.msg_attr.user[0] == 0)
 	return (bounce_append(BOUNCE_FLAGS(state.request),
-			      BOUNCE_ATTR(state.msg_attr)));
-    }
+			      BOUNCE_ATTR(state.msg_attr, "5.1.3"),
+			  "null username in %s", state.msg_attr.recipient));
 
     /*
      * Run the recipient through the delivery switch.
