@@ -819,6 +819,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
     NOCLOBBER int mail_from_rejected;
     NOCLOBBER int downgrading;
     int     mime_errs;
+    MIME_STATE_DETAIL *detail;
 
     /*
      * Macros for readability.
@@ -1378,9 +1379,9 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 					  vstring_str(session->scratch),
 					  VSTRING_LEN(session->scratch));
 		    if (mime_errs) {
-			smtp_mesg_fail(state, "5.6.5", 554,
-				       "MIME 7-bit conversion failed: %s",
-				       mime_state_error(mime_errs));
+			detail = mime_state_detail(mime_errs);
+			smtp_mesg_fail(state, detail->dsn, 554, "%s",
+				       detail->text);
 			RETURN(0);
 		    }
 		}
@@ -1396,13 +1397,14 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 		 * ending in newline via /usr/sbin/sendmail while MIME input
 		 * processing is turned off, and MIME 8bit->7bit conversion
 		 * is requested upon delivery.
+		 * 
+		 * Or some error while doing generic address mapping.
 		 */
 		mime_errs =
 		    mime_state_update(session->mime_state, rec_type, "", 0);
 		if (mime_errs) {
-		    smtp_mesg_fail(state, "5.6.5", 554,
-				   "MIME 7-bit conversion failed: %s",
-				   mime_state_error(mime_errs));
+		    detail = mime_state_detail(mime_errs);
+		    smtp_mesg_fail(state, detail->dsn, 554, "%s", detail->text);
 		    RETURN(0);
 		}
 	    } else if (prev_type == REC_TYPE_CONT)	/* missing newline */
