@@ -35,7 +35,7 @@
 /*	VSTREAM *stream;
 /*	int	rec_type;
 /*	const char *data;
-/*	ssize_t	len;
+/*	int	len;
 /*
 /*	int	smtpd_proxy_rec_fprintf(stream, rec_type, format, ...)
 /*	VSTREAM *stream;
@@ -140,10 +140,6 @@
 #include <sys_defs.h>
 #include <ctype.h>
 
-#ifdef STRCASECMP_IN_STRINGS_H
-#include <strings.h>
-#endif
-
 /* Utility library. */
 
 #include <msg.h>
@@ -161,8 +157,6 @@
 #include <mail_params.h>
 #include <rec_type.h>
 #include <mail_proto.h>
-#include <mail_params.h>		/* null_format_string */
-#include <xtext.h>
 
 /* Application-specific. */
 
@@ -184,7 +178,7 @@
   */
 #define STR(x)	vstring_str(x)
 #define LEN(x)	VSTRING_LEN(x)
-#define SMTPD_PROXY_CONNECT null_format_string
+#define SMTPD_PROXY_CONNECT ((char *) 0)
 #define STREQ(x, y)	(strcmp((x), (y)) == 0)
 
 /* smtpd_xforward_flush - flush forwarding information */
@@ -213,20 +207,12 @@ static int smtpd_xforward(SMTPD_STATE *state, VSTRING *buf, const char *name,
 #define CONSTR_LEN(s)	(sizeof(s) - 1)
 #define PAYLOAD_LIMIT	(512 - CONSTR_LEN("250 " XFORWARD_CMD "\r\n"))
 
+    /*
+     * How much space does this attribute need?
+     */
     if (!value_available)
 	value = XFORWARD_UNAVAILABLE;
-
-    /*
-     * Encode the attribute value.
-     */
-    if (state->expand_buf == 0)
-	state->expand_buf = vstring_alloc(100);
-    xtext_quote(state->expand_buf, value, "");
-
-    /*
-     * How much space does this attribute need? SPACE name = value.
-     */
-    new_len = strlen(name) + strlen(STR(state->expand_buf)) + 2;
+    new_len = strlen(name) + strlen(value) + 2;	/* SPACE name = value */
     if (new_len > PAYLOAD_LIMIT)
 	msg_warn("%s command payload %s=%.10s... exceeds SMTP protocol limit",
 		 XFORWARD_CMD, name, value);
@@ -237,7 +223,7 @@ static int smtpd_xforward(SMTPD_STATE *state, VSTRING *buf, const char *name,
     if (VSTRING_LEN(buf) > 0 && VSTRING_LEN(buf) + new_len > PAYLOAD_LIMIT)
 	if ((ret = smtpd_xforward_flush(state, buf)) < 0)
 	    return (ret);
-    vstring_sprintf_append(buf, " %s=%s", name, STR(state->expand_buf));
+    vstring_sprintf_append(buf, " %s=%s", name, value);
 
     return (0);
 }
@@ -263,8 +249,6 @@ int     smtpd_proxy_open(SMTPD_STATE *state, const char *service,
 	0, 0,
     };
     CLEANUP_STAT_DETAIL *detail;
-    int     (*connect_fn) (const char *, int, int);
-    const char *endpoint;
 
     /*
      * This buffer persists beyond the end of a proxy session so we can
@@ -273,24 +257,10 @@ int     smtpd_proxy_open(SMTPD_STATE *state, const char *service,
     if (state->proxy_buffer == 0)
 	state->proxy_buffer = vstring_alloc(10);
 
-    /*
-     * Find connection method (default inet)
-     */
-    if (strncasecmp("unix:", service, 5) == 0) {
-	endpoint = service + 5;
-	connect_fn = unix_connect;
-    } else {
-	if (strncasecmp("inet:", service, 5) == 0)
-	    endpoint = service + 5;
-	else
-	    endpoint = service;
-	connect_fn = inet_connect;
-    }
-
     /*
      * Connect to proxy.
      */
-    if ((fd = connect_fn(endpoint, BLOCKING, timeout)) < 0) {
+    if ((fd = inet_connect(service, BLOCKING, timeout)) < 0) {
 	state->error_mask |= MAIL_ERROR_SOFTWARE;
 	state->err |= CLEANUP_STAT_PROXY;
 	msg_warn("connect to proxy service %s: %m", service);
@@ -358,31 +328,26 @@ int     smtpd_proxy_open(SMTPD_STATE *state, const char *service,
     if (state->proxy_xforward_features) {
 	buf = vstring_alloc(100);
 	bad = 0;
-	if (state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_NAME)
-	    bad = smtpd_xforward(state, buf, XFORWARD_NAME,
-				 IS_AVAIL_CLIENT_NAME(FORWARD_NAME(state)),
-				 FORWARD_NAME(state));
-	if (bad == 0
-	    && (state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_ADDR))
-	    bad = smtpd_xforward(state, buf, XFORWARD_ADDR,
-				 IS_AVAIL_CLIENT_ADDR(FORWARD_ADDR(state)),
-				 FORWARD_ADDR(state));
-	if (bad == 0
-	    && (state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_HELO))
-	    bad = smtpd_xforward(state, buf, XFORWARD_HELO,
-				 IS_AVAIL_CLIENT_HELO(FORWARD_HELO(state)),
-				 FORWARD_HELO(state));
-	if (bad == 0
-	    && (state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_PROTO))
-	    bad = smtpd_xforward(state, buf, XFORWARD_PROTO,
-				 IS_AVAIL_CLIENT_PROTO(FORWARD_PROTO(state)),
-				 FORWARD_PROTO(state));
-	if (bad == 0
-	  && (state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_DOMAIN))
-	    bad = smtpd_xforward(state, buf, XFORWARD_DOMAIN, 1,
+	if ((!(state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_NAME)
+	     || !(bad = smtpd_xforward(state, buf, XFORWARD_NAME,
+				  IS_AVAIL_CLIENT_NAME(FORWARD_NAME(state)),
+				       FORWARD_NAME(state))))
+	    && (!(state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_ADDR)
+		|| !(bad = smtpd_xforward(state, buf, XFORWARD_ADDR,
+				  IS_AVAIL_CLIENT_ADDR(FORWARD_ADDR(state)),
+					  FORWARD_ADDR(state))))
+	    && (!(state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_HELO)
+		|| !(bad = smtpd_xforward(state, buf, XFORWARD_HELO,
+				  IS_AVAIL_CLIENT_HELO(FORWARD_HELO(state)),
+					  FORWARD_HELO(state))))
+	  && (!(state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_PROTO)
+	      || !(bad = smtpd_xforward(state, buf, XFORWARD_PROTO,
+				IS_AVAIL_CLIENT_PROTO(FORWARD_PROTO(state)),
+					FORWARD_PROTO(state))))
+	 && (!(state->proxy_xforward_features & SMTPD_PROXY_XFORWARD_DOMAIN)
+	     || !(bad = smtpd_xforward(state, buf, XFORWARD_DOMAIN, 1,
 			 STREQ(FORWARD_DOMAIN(state), MAIL_ATTR_RWR_LOCAL) ?
-				 XFORWARD_DOM_LOCAL : XFORWARD_DOM_REMOTE);
-	if (bad == 0)
+				XFORWARD_DOM_LOCAL : XFORWARD_DOM_REMOTE))))
 	    bad = smtpd_xforward_flush(state, buf);
 	vstring_free(buf);
 	if (bad) {
@@ -461,7 +426,7 @@ int     smtpd_proxy_cmd(SMTPD_STATE *state, int expect, const char *fmt,...)
     if (vstream_ftimeout(state->proxy)
 	|| vstream_ferror(state->proxy)
 	|| vstream_feof(state->proxy)
-	|| ((err = vstream_setjmp(state->proxy)) != 0
+	|| ((err = vstream_setjmp(state->proxy) != 0)
 	    && smtpd_proxy_rdwr_error(state->proxy, err))) {
 	state->error_mask |= MAIL_ERROR_SOFTWARE;
 	state->err |= CLEANUP_STAT_PROXY;
@@ -572,7 +537,7 @@ int     smtpd_proxy_cmd(SMTPD_STATE *state, int expect, const char *fmt,...)
 /* smtpd_proxy_rec_put - send message content, rec_put() clone */
 
 int     smtpd_proxy_rec_put(VSTREAM *stream, int rec_type,
-			            const char *data, ssize_t len)
+			            const char *data, int len)
 {
     int     err;
 
